import { mapKeys, toPairs, isUndefined, isString, partial } from 'lodash';
import request, { normParams } from '../utils/request';
import { split } from '../utils/filter';

const parseFilters = filtersIn => (filtersIn || []).filter(({ filter }) => !!filter).map(({ key, filter }) => [
  split(key, false).map(value => `f-${value}`).join('|'),
  filter,
]);

export const datasourceApi = (coordinate) => {
  const { containerType, containerName, datasourceName } = isString(coordinate) ? {
    containerType: 'global',
    moduleName: coordinate,
  } : (coordinate || {});
  if (containerType === 'global') {
    return {
      query: partial(calcGlobalDatasource, datasourceName),
      count: partial(countGlobalDatasource, datasourceName),
      meta: partial(getGlobalDatasourceMeta, datasourceName),
    };
  } else if (containerType === 'module') {
    return {
      query: partial(calcModuleDatasource, containerName, datasourceName),
      count: partial(countModuleDatasource, containerName, datasourceName),
      meta: partial(getModuleDatasourceMeta, containerName, datasourceName),
    };
  } else {
    throw new Error(`Unsupported containerType: ${containerType}`);
  }
};

const makeQueryParams = ({ pst, psz, filters = [], sortBys = [], sortTypes = [], params = {} }) => {
  let stBy = sortBys.join(',');
  stBy = stBy || undefined;
  let stType = sortTypes.join(',');
  stType = stType || undefined;
  return [
    ...toPairs({ pst, psz }),
    ...parseFilters(filters),
    ['stBy', stBy],
    ['stType', stType],
    ...normParams(mapKeys(params, (v, k) => `p-${k}`)),
  ].filter(v => v && !isUndefined((v[1])));
};

export async function calcGlobalDatasource(name, { pst, psz, filters = [], sortBys = [], sortTypes = [], params = {} }) {
  return request(`/api/datasource/${name}`, makeQueryParams({ pst, psz, filters, sortBys, sortTypes, params }));
}

export async function countGlobalDatasource(name, { filters = [], params = {} }) {
  return request(`/api/datasource/${name}/count`, makeQueryParams({ filters, params }));
}

export async function getGlobalDatasourceMeta(name) {
  return request(`/api/datasource/${name}`);
}

export async function calcModuleDatasource(mdName, dsName, { pst, psz, filters = [], sortBys = [], sortTypes = [], params = {} }) {
  return request(`/api/modules/module/${mdName}/datasource/${dsName}`, makeQueryParams({ pst, psz, filters, sortBys, sortTypes, params }));
}

export async function countModuleDatasource(mdName, dsName, { filters = [], params = {} }) {
  return request(`/api/modules/module/${mdName}/datasource/${dsName}/count`, makeQueryParams({ filters, params }));
}

export async function getModuleDatasourceMeta(mdName, dsName) {
  return request(`/api/modules/module/${mdName}/datasource/${dsName}/meta`);
}
